# audioopy.aio module

## List of classes

## Class `WaveIO`

### Description

*Opening and saving Waveform Audio File Format (WAV) files.*

It extends the AudioPCM class to provide functionality for opening and
saving audio files in the Waveform Audio File Format (WAV).
Waveform Audio File Format is a Microsoft and IBM audio file format
standard for storing an audio bitstream on PCs. It is an application of
the Resource Interchange File Format (RIFF) bitstream format method for
storing data in "chunks".

##### Example

    >>> # Create an instance and open the WAV file
    >>> wave_io = WaveIO()
    >>> wave_io.open("input.wav")
    >>> # Save the audio content to a new WAV file
    >>> wave_io.save("output.wav")
    >>> # Save a fragment of the audio content to a new WAV file
    >>> frames = wave_io.read_frames(1000)
    >>> wave_io.save_fragment("fragment.wav", frames)


### Constructor

#### __init__

```python
def __init__(self):
    """Create a WaveIO instance."""
    super(WaveIO, self).__init__()
```

*Create a WaveIO instance.*



### Public functions

#### open

```python
def open(self, filename: str) -> None:
    """Get an audio from a Waveform Audio File Format file.

        :param filename (str) input file name.
        :raises: wave.Error: if the file cannot be opened.

        """
    self._audio_fp = wave.open(filename, 'r')
```

*Get an audio from a Waveform Audio File Format file.*

##### Parameters

filename(*str*) input file name.


##### Raises

- *wave.Error*: if the file cannot be opened.

#### save

```python
def save(self, filename: str) -> None:
    """Write an audio content as a Waveform Audio File Format file.

        :param filename (str) output filename.

        """
    if self._audio_fp is not None:
        self.rewind()
        frames = self._audio_fp.readframes(self._audio_fp.getnframes())
        self.save_fragment(filename, frames)
    elif len(self._channels) == 1:
        channel = self._channels[0]
        f = wave.Wave_write(filename)
        f.setnchannels(1)
        f.setsampwidth(channel.get_sampwidth())
        f.setframerate(channel.get_framerate())
        try:
            f.writeframes(channel.get_frames())
        finally:
            f.close()
    else:
        self.verify_channels()
        sw = self._channels[0].get_sampwidth()
        frames = b''
        for i in range(0, self._channels[0].get_nframes() * sw, sw):
            for j in range(len(self._channels)):
                frames += self._channels[j].get_frames(sw)
        f = wave.Wave_write(filename)
        f.setnchannels(len(self._channels))
        f.setsampwidth(self._channels[0].get_sampwidth())
        f.setframerate(self._channels[0].get_framerate())
        try:
            f.writeframes(frames)
        finally:
            f.close()
```

*Write an audio content as a Waveform Audio File Format file.*

##### Parameters

filename(*str*) output filename.

#### save_fragment

```python
def save_fragment(self, filename: str, frames: bytes) -> None:
    """Write an audio content as a Waveform Audio File Format file.

        :param filename: (str) output filename.
        :param frames: (bytes) the frames to write

        """
    f = wave.Wave_write(filename)
    f.setnchannels(self.get_nchannels())
    f.setsampwidth(self.get_sampwidth())
    f.setframerate(self.get_framerate())
    try:
        f.writeframes(frames)
    finally:
        f.close()
```

*Write an audio content as a Waveform Audio File Format file.*

##### Parameters

- **filename**: (*str*) output filename.
- **frames**: (*bytes*) the frames to write





~ Created using [Clamming](https://clamming.sf.net) version 1.9 ~
