# makedoc.py
# Summary: Create the documentation of AudiooPy, using Clamming library.
# Usage: python makedoc.py
#
# This file is part of AudiooPy tool.
# (C) 2024 Brigitte Bigi, Laboratoire Parole et Langage,
# Aix-en-Provence, France.
#
# Use of this software is governed by the GNU Affero Public License, version 3.
#
# AudiooPy is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# AudiooPy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with AudiooPy. If not, see <http://www.gnu.org/licenses/>.
#
# This banner notice must not be removed.
#
# ---------------------------------------------------------------------------

import sys
import logging

import audioopy
import audioopy.aio
import audioopy.ipus
try:
    from clamming import ClamsModules
    from clamming import ExportOptions
except ImportError:
    print("This program requires `Clamming` documentation generator.")
    print("It can be installed with: pip install ClammingPy.")
    print("See <https://pypi.org/project/Clamming/> for details.")
    sys.exit(-1)

# ---------------------------------------------------------------------------
logging.getLogger().setLevel(0)

# -------------------------------------------------
# List of modules to be documented: automatically create the documentation of
# all known classes of the following 'audioopy' packages.
# -------------------------------------------------
packages = list()
packages.append(audioopy.aio)
packages.append(audioopy)
packages.append(audioopy.ipus)

# ----------------------------
# Options for exportation
# ----------------------------
opts_export = ExportOptions()
opts_export.software = 'AudiooPy ' + audioopy.__version__
opts_export.copyright = audioopy.__copyright__
opts_export.url = 'https://audioopy.sourceforge.io'
opts_export.title = 'AudiooPy doc'
# ... statics is the relative path to a folder with the CSS, JS, etc.
opts_export.wexa_statics = './wexa_statics'
opts_export.statics = './statics'
# ... the theme must correspond to a statics/<theme>.css file
opts_export.theme = 'light'
# ... the favicon and icon are files in the statics folder
opts_export.favicon = 'audioopy32x32.ico'
opts_export.icon = 'audioopy.png'
opts_export.readme = True

# -------------------------------------------------
# Generate documentation
# -------------------------------------------------
clams_modules = ClamsModules(packages)

# Export documentation into HTML files.
# One .html file = one documented class.
clams_modules.html_export_packages("docs", opts_export, "README.md")

# Export documentation into a Markdown file.
# One .md file = one documented module.
clams_modules.markdown_export_packages("docs", opts_export)
