"""
:filename: tests.test_audioconverter.py
:author: Nicolas Chazeau, Brigitte Bigi
:contact: contact@sppas.org
:summary: Test the conversion of audio.

.. _This file is part of AudiooPy:
..
    ---------------------------------------------------------------------

    Copyright (C) 2024 Brigitte Bigi
    Laboratoire Parole et Langage, Aix-en-Provence, France

    Use of this software is governed by the GNU Affero Public License, version 3.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program. If not, see <https://www.gnu.org/licenses/>.

    This banner notice must not be removed.

    ---------------------------------------------------------------------

"""

import unittest
import os

from audioopy.aio import open as audio_open
from audioopy.audioconvert import AudioConverter

sample_1 = os.path.join(os.path.dirname(os.path.abspath(__file__)), "samples", "oriana1.wav")
sample_2 = os.path.join(os.path.dirname(os.path.abspath(__file__)), "samples", "F_F_B003_P8.wav")
sample_3 = os.path.join(os.path.dirname(os.path.abspath(__file__)), "samples", "oriana3.wave")

# ---------------------------------------------------------------------------


class TestAudioUtils(unittest.TestCase):

    def setUp(self):
        self._sample_1 = audio_open(sample_1)
        self._sample_2 = audio_open(sample_2)
        self._sample_3 = audio_open(sample_3)

    def tearDown(self):
        self._sample_1.close()
        self._sample_2.close()
        self._sample_3.close()

    def test_Samples2Frames(self):
        s1 = self._sample_1.read_samples(10)
        s2 = self._sample_2.read_samples(10)
        s3 = self._sample_3.read_samples(10)
        self._sample_1.rewind()
        self._sample_2.rewind()
        self._sample_3.rewind()
        f1 = AudioConverter().samples2frames(s1, self._sample_1.get_sampwidth(), self._sample_1.get_nchannels())
        f2 = AudioConverter().samples2frames(s2, self._sample_2.get_sampwidth(), self._sample_2.get_nchannels())
        f3 = AudioConverter().samples2frames(s3, self._sample_3.get_sampwidth(), self._sample_3.get_nchannels())

        self.assertEqual(self._sample_1.read_frames(10), f1)
        self.assertEqual(self._sample_2.read_frames(10), f2)
        self.assertEqual(self._sample_3.read_frames(10), f3)

        self.assertEqual(s1, [[1, -1, 1, 1, 0, 0, -1, 1, -1, 1]])
        self.assertEqual(s2, [[-26, -54, -57, -70, -56, -54, -28, -33, -14, 1]])
        self.assertEqual(s3, [[0, 1, 1, 0, 1, -2, 2, -1, 1, -2], [0, 1, -5, 5, -8, 14, -76, -169, -139, -149]])

    def test_Frames2Samples(self):
        f1 = self._sample_1.read_frames(10)
        self._sample_1.rewind()
        f2 = self._sample_2.read_frames(10)
        self._sample_2.rewind()
        f3 = self._sample_3.read_frames(10)
        self._sample_3.rewind()
        s1 = AudioConverter().unpack_data(f1, self._sample_1.get_sampwidth(), self._sample_1.get_nchannels())
        s2 = AudioConverter().unpack_data(f2, self._sample_2.get_sampwidth(), self._sample_2.get_nchannels())
        s3 = AudioConverter().unpack_data(f3, self._sample_2.get_sampwidth(), self._sample_3.get_nchannels())

        self.assertEqual(s1, [[1, -1, 1, 1, 0, 0, -1, 1, -1, 1]])
        self.assertEqual(s2, [[-26, -54, -57, -70, -56, -54, -28, -33, -14, 1]])
        self.assertEqual(s3, [[0, 1, 1, 0, 1, -2, 2, -1, 1, -2], [0, 1, -5, 5, -8, 14, -76, -169, -139, -149]])

    def test_frames_and_samples(self):
        f1 = self._sample_1.read_frames(10)
        self._sample_1.rewind()
        f2 = self._sample_2.read_frames(10)
        self._sample_2.rewind()
        f3 = self._sample_3.read_frames(10)
        self._sample_3.rewind()

        s1 = AudioConverter().unpack_data(f1, self._sample_1.get_sampwidth(), self._sample_1.get_nchannels())
        s2 = AudioConverter().unpack_data(f2, self._sample_2.get_sampwidth(), self._sample_2.get_nchannels())
        s3 = AudioConverter().unpack_data(f3, self._sample_2.get_sampwidth(), self._sample_3.get_nchannels())
        f1c = AudioConverter().samples2frames(s1, self._sample_1.get_sampwidth(), self._sample_1.get_nchannels())
        f2c = AudioConverter().samples2frames(s2, self._sample_2.get_sampwidth(), self._sample_2.get_nchannels())
        f3c = AudioConverter().samples2frames(s3, self._sample_3.get_sampwidth(), self._sample_3.get_nchannels())

        self.assertEqual(f1, f1c)
        self.assertEqual(f2, f2c)
        self.assertEqual(f3, f3c)

    def test_db(self):
        """Test amp2db."""
        self.assertEqual(70, AudioConverter().amp2db(3162))
        self.assertEqual(46.02, AudioConverter().amp2db(200))
        self.assertEqual(40, AudioConverter().amp2db(100))
        self.assertEqual(10, AudioConverter().amp2db(3.162))
        self.assertEqual(9.54, AudioConverter().amp2db(3))
        self.assertEqual(0., AudioConverter().amp2db(2))

    def test_mel(self):
        """Test hz2mel/mel2hz."""
        hz = 200
        mel = AudioConverter().hz2mel(hz)
        rehz = AudioConverter().mel2hz(mel)
        self.assertEqual(hz, rehz)
