# -*- coding:utf-8 -*-
"""
:filename: tests.test_audioframes.py
:author: Brigitte Bigi
:contact: contact@sppas.org
:summary: Test of the audio frames.

.. _This file is part of AudiooPy:
..
    ---------------------------------------------------------------------

    Copyright (C) 2024 Brigitte Bigi
    Laboratoire Parole et Langage, Aix-en-Provence, France

    Use of this software is governed by the GNU Affero Public License, version 3.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program. If not, see <https://www.gnu.org/licenses/>.

    This banner notice must not be removed.

    ---------------------------------------------------------------------

"""

import unittest
import os.path

from audioopy.audioopyexc import NumberFramesError
from audioopy.audioopyexc import SampleWidthError
from audioopy.audioopyexc import ChannelIndexError
from audioopy.audioframes import AudioFrames
from audioopy.aio import open as audio_open

PROGRAM = os.path.dirname(os.path.abspath(__file__))
sample_1 = os.path.join(PROGRAM, "samples", "oriana1.wav")      # mono - 16kHz
sample_2 = os.path.join(PROGRAM, "samples", "oriana3.wave")     # stereo - 16kHz
sample_3 = os.path.join(PROGRAM, "samples", "F_F_B003_P8.wav")  # mono - 44.1kHz
sample_4 = os.path.join(PROGRAM, "samples", "ENG_M15_ENG_T02.wav")  # mono - 48kHz

# ---------------------------------------------------------------------------


class TestAudioFrames(unittest.TestCase):

    def setUp(self):
        self._sample_1 = audio_open(sample_1)
        self._sample_2 = audio_open(sample_2)
        self._sample_3 = audio_open(sample_3)
        self.frames_1 = self._sample_1.read_frames(self._sample_1.get_nframes())
        self.frames_2 = self._sample_2.read_frames(self._sample_2.get_nframes())
        self.frames_3 = self._sample_3.read_frames(self._sample_3.get_nframes())
        self._sample_1.rewind()
        self._sample_2.rewind()
        self._sample_3.rewind()

    # -----------------------------------------------------------------------

    def test_init(self):
        a = AudioFrames()
        self.assertEqual(a._frames, b"")
        self.assertEqual(a._sampwidth, 2)
        self.assertEqual(a._nchannels, 1)

        frames = b"\x01\x00"
        a = AudioFrames(frames, sampwidth=2, nchannels=1)
        self.assertEqual(a._frames, frames)
        self.assertEqual(a._sampwidth, 2)
        self.assertEqual(a._nchannels, 1)

        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00'
        a = AudioFrames(frames, sampwidth=2, nchannels=2)
        self.assertEqual(a._frames, frames)
        self.assertEqual(a._sampwidth, 2)
        self.assertEqual(a._nchannels, 2)

        # invalid size
        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00'
        with self.assertRaises(SampleWidthError):
            AudioFrames(frames, sampwidth=3, nchannels=1)

        # invalid nb of channels
        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00'
        with self.assertRaises(ChannelIndexError):
            AudioFrames(frames, sampwidth=2, nchannels=-1)

        # inconsistencies
        frames = b'\x01\x00\x02\x00\x03\x00\x04'
        with self.assertRaises(NumberFramesError):
            AudioFrames(frames, sampwidth=2, nchannels=2)

        frames = b"\x01\x00"
        with self.assertRaises(NumberFramesError):
            AudioFrames(frames, sampwidth=2, nchannels=2)

    # -----------------------------------------------------------------------
    # Tests functions that return values
    # -----------------------------------------------------------------------

    def test_getsample(self):
        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00'

        # 8bits mono
        audio = AudioFrames(frames, sampwidth=1, nchannels=1)
        for i in range(len(frames)):
            if i % 2 == 0:
                self.assertEqual((i//2) + 1, audio.get_sample(i))
            else:
                self.assertEqual(0, audio.get_sample(i))

        # 16bits mono: the most frequent format in corpus recordings
        audio = AudioFrames(frames, sampwidth=2, nchannels=1)
        for i in range(len(frames) // 2):
            self.assertEqual(i + 1, audio.get_sample(i))

        # 16bits stereo
        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00\x06\x00'
        audio = AudioFrames(frames, sampwidth=2, nchannels=2)
        for i in range(len(frames) // 2):
            self.assertEqual(i + 1, audio.get_sample(i))

        # empty frames
        audio = AudioFrames(b'', 2, 2)
        with self.assertRaises(Exception):
            self.assertEqual(0., audio.get_sample(2))

    # -----------------------------------------------------------------------

    def test_minmax(self):
        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00'

        audio = AudioFrames(frames, 2, 1)
        self.assertEqual((1, 5), audio.minmax())

        audio = AudioFrames(frames, 1, 1)
        self.assertEqual((0, 5), audio.minmax())

        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00\x06\x00'
        audio = AudioFrames(frames, 2, 2)
        self.assertEqual((1, 6), audio.minmax())

        audio = AudioFrames(b'', 2, 2)
        self.assertEqual((0, 0), audio.minmax())

        # Real audios files
        self.assertEqual((-8613, 9405), AudioFrames(self.frames_1, self._sample_1.get_sampwidth()).minmax())
        self.assertEqual((-9340, 9401), AudioFrames(self.frames_2, self._sample_2.get_sampwidth()).minmax())
        self.assertEqual((-32768, 26789), AudioFrames(self.frames_3, self._sample_3.get_sampwidth()).minmax())

    # -----------------------------------------------------------------------

    def test_avg(self):
        # Real audios files
        self.assertEqual(-0.55, AudioFrames(self.frames_1, self._sample_1.get_sampwidth()).avg())
        self.assertEqual(-0.54, AudioFrames(self.frames_2, self._sample_2.get_sampwidth()).avg())
        self.assertEqual(-0.03, AudioFrames(self.frames_3, self._sample_3.get_sampwidth()).avg())

        # With audioop (Python < 3.13) a 'long' was returned:
        #self.assertEqual(-1, sppasAudioFrames(self.frames_1, self._sample_1.get_sampwidth()).avg())
        #self.assertEqual(-1, sppasAudioFrames(self.frames_2, self._sample_2.get_sampwidth()).avg())
        #self.assertEqual(-1, sppasAudioFrames(self.frames_3, self._sample_3.get_sampwidth()).avg())

        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00'

        audio = AudioFrames(frames, 2, 1)
        self.assertEqual(3.0, audio.avg())

        # different_sample_widths
        audio = AudioFrames(frames, 1, 1)
        self.assertEqual(1.5, audio.avg())

        # different_number_of_channels
        with self.assertRaises(NumberFramesError):
            AudioFrames(frames, 2, 2)

        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00\x06\x00'
        audio = AudioFrames(frames, 2, 2)
        self.assertEqual(3.5, audio.avg())

        # empty frames
        audio = AudioFrames(b'', 2, 2)
        self.assertEqual(0., audio.avg())

    # -----------------------------------------------------------------------

    def test_rms(self):
        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00'

        audio = AudioFrames(frames, 2, 1)
        self.assertEqual(3.32, audio.rms())

        audio = AudioFrames(frames, 1, 1)
        self.assertEqual(2.35, audio.rms())

        # Two channels
        audio = AudioFrames(b'\x01\x00\x02\x00\x03\x00\x04\x00', 2, 2)
        self.assertEqual(audio.rms(), 2.74)

        # empty frames
        audio = AudioFrames(b'', 2, 2)
        self.assertEqual(0, audio.rms())

        # Real audios files
        self.assertEqual(696.02, AudioFrames(self.frames_1, self._sample_1.get_sampwidth()).rms())
        self.assertEqual(708.98, AudioFrames(self.frames_2, self._sample_2.get_sampwidth()).rms())
        self.assertEqual(2863.16, AudioFrames(self.frames_3, self._sample_3.get_sampwidth()).rms())

    # -----------------------------------------------------------------------

    def test_cross(self):
        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00'

        audio = AudioFrames(frames, 2, 1)
        self.assertEqual(0, audio.cross())

        audio = AudioFrames(frames, 1, 1)
        self.assertEqual(0, audio.cross())

        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00\x06\x00'
        audio = AudioFrames(frames, 2, 2)
        self.assertEqual(0, audio.cross())

        audio = AudioFrames(b'', 2, 2)
        self.assertEqual(-1, audio.cross())

        # Real audios files
        self.assertEqual(49827, AudioFrames(self.frames_1, self._sample_1.get_sampwidth()).cross())
        self.assertEqual(283794, AudioFrames(self.frames_2, self._sample_2.get_sampwidth()).cross())

    def test_clipping_rate(self):
        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00'

        audio = AudioFrames(frames, 2, 1)
        self.assertEqual(0., audio.clipping_rate(0.1))

        audio = AudioFrames(frames, 1, 1)
        self.assertEqual(0., audio.clipping_rate(0.1))

        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00\x06\x00'
        audio = AudioFrames(frames, 2, 2)
        self.assertEqual(0., audio.clipping_rate(0.1))

        audio = AudioFrames(b'', 2, 2)
        self.assertEqual(0., audio.clipping_rate(0.1))

        # Real audios files
        self.assertEqual(0.008726, round(AudioFrames(self.frames_1, self._sample_1.get_sampwidth()).clipping_rate(0.1), 6))
        self.assertEqual(0.009585, round(AudioFrames(self.frames_2, self._sample_2.get_sampwidth()).clipping_rate(0.1), 6))

    # -----------------------------------------------------------------------
    # Tests functions that return frames
    # -----------------------------------------------------------------------

    def test_mul(self):
        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00'

        audio = AudioFrames(frames, 2, 1)
        self.assertEqual(b'\x02\x00\x04\x00\x06\x00\x08\x00\n\x00', audio.mul(2.0))

        audio = AudioFrames(frames, 1, 1)
        self.assertEqual(b'\x04\x00\x08\x00\x0c\x00\x10\x00\x14\x00', audio.mul(4.))

        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00\x06\x00'
        audio = AudioFrames(frames, 2, 2)
        self.assertEqual(b'\x02\x00\x04\x00\x06\x00\x08\x00\n\x00\x0c\x00', audio.mul(2.))

        audio = AudioFrames(b'', 2, 2)
        self.assertEqual(b'', audio.mul(2.2))

        # Real audios files
        mul_1 = AudioFrames(self.frames_1, self._sample_1.get_sampwidth()).mul(2.0)
        self.assertTrue(mul_1.startswith(b'\x02\x00\xfe\xff\x02\x00\x02\x00\x00\x00\x00\x00\xfe\xff\x02\x00\xfe\xff\x02\x00'))
        mul_2 = AudioFrames(self.frames_2, self._sample_2.get_sampwidth()).mul(2.0)
        self.assertTrue(mul_2.startswith(b'\x00\x00\x00\x00\x02\x00\x02\x00\x02\x00\xf6\xff\x00\x00\n\x00\x02\x00\xf0\xff'))

    # -----------------------------------------------------------------------

    def test_bias(self):
        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00'

        audio = AudioFrames(frames, 2, 1)
        self.assertEqual(b'\x03\x00\x04\x00\x05\x00\x06\x00\x07\x00', audio.bias(2))

        audio = AudioFrames(frames, 1, 1)
        self.assertEqual(b'\x05\x04\x06\x04\x07\x04\x08\x04\t\x04', audio.bias(4))

        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00\x06\x00'
        audio = AudioFrames(frames, 2, 2)
        self.assertEqual(b'\x03\x00\x04\x00\x05\x00\x06\x00\x07\x00\x08\x00', audio.bias(2))

        audio = AudioFrames(b'', 2, 2)
        self.assertEqual(b'', audio.bias(2))

        # Real audios files
        mul_1 = AudioFrames(self.frames_1, self._sample_1.get_sampwidth()).bias(2)
        self.assertTrue(mul_1.startswith(b'\x03\x00\x01\x00\x03\x00\x03\x00\x02\x00\x02\x00\x01\x00\x03\x00\x01\x00\x03\x00'))
        mul_2 = AudioFrames(self.frames_2, self._sample_2.get_sampwidth()).bias(2)
        self.assertTrue(mul_2.startswith(b'\x02\x00\x02\x00\x03\x00\x03\x00\x03\x00\xfd\xff\x02\x00\x07\x00\x03\x00\xfa\xff'))

    # -----------------------------------------------------------------------

    def test_sampwidth(self):
        # Tested frames:
        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00'
        # Expected results:
        one_to_two = b'\x00\x01\x00\x00\x00\x02\x00\x00\x00\x03\x00\x00\x00\x04\x00\x00\x00\x05\x00\x00'
        one_to_four = (b'\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x02\x00\x00\x00\x00'
                       b'\x00\x00\x00\x03\x00\x00\x00\x00\x00\x00\x00\x04\x00\x00\x00\x00'
                       b'\x00\x00\x00\x05\x00\x00\x00\x00')
        two_to_one = b'\x00\x00\x00\x00\x00'
        two_to_four = b'\x00\x00\x01\x00\x00\x00\x02\x00\x00\x00\x03\x00\x00\x00\x04\x00\x00\x00\x05\x00'

        # One channel; size 1 -> 2 ; 1 -> 4
        audio = AudioFrames(frames, 1, 1)
        self.assertEqual(frames, audio.change_sampwidth(1))
        #self.assertEqual(one_to_two, audio.change_sampwidth(2))
        #self.assertEqual(one_to_four, audio.change_sampwidth(4))

        # One channel; size 2 -> 1 ; 2 -> 4
        audio = AudioFrames(frames, 2, 1)
        #self.assertEqual(two_to_one, audio.change_sampwidth(1))
        #self.assertEqual(frames, audio.change_sampwidth(2))
        #self.assertEqual(two_to_four, audio.change_sampwidth(4))

        # One channel; size 4 -> 1 ; 4 -> 2
        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00'
        audio = AudioFrames(frames, 4, 1)
        #self.assertEqual(b'\x00\x00', audio.change_sampwidth(1))
        #self.assertEqual(b'\x02\x00\x04\x00', audio.change_sampwidth(2))
        #self.assertEqual(frames, audio.change_sampwidth(4))

        # Two channels
        #audio = AudioFrames(frames, 2, 2)
        #self.assertEqual(b'\x00\x00\x00\x00', audio.change_sampwidth(1))

        # No signal.
        audio = AudioFrames(b'', 2, 2)
        self.assertEqual(b'', audio.change_sampwidth(1))

        # Real audios files
        res_1 = AudioFrames(self.frames_1, self._sample_1.get_sampwidth()).change_sampwidth(1)
        self.assertTrue(res_1.startswith(b'\x00\xff\x00\x00\x00\x00\xff\x00\xff\x00\xff\x00\x00\x00\xff\x00\xff\xff\x00\xff'))
        res_2 = AudioFrames(self.frames_2, self._sample_2.get_sampwidth()).change_sampwidth(4)
        self.assertTrue(res_2.startswith(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x01\x00\x00\x00\x01\x00'))

    # -----------------------------------------------------------------------

    def test_resample(self):
        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00'

        audio = AudioFrames(frames, 2, 1)
        self.assertEqual(frames, audio.resample(16000, 16000))
        self.assertEqual(b'\x01\x00\x03\x00\x05\x00', audio.resample(32000, 16000))
        self.assertEqual(b'\x02\x00\x04\x00', audio.resample(48000, 16000))

        audio = AudioFrames(frames, 2, 1)
        self.assertEqual(b'\x01\x00\x03\x00', audio.resample(44100, 16000))

        audio = AudioFrames(frames, 1, 1)
        self.assertEqual(b'\x01\x00\x02\x03', audio.resample(44100, 16000))

        audio = AudioFrames(b'\x01\x00\x02\x00\x03\x00\x04\x00', 2, 2)
        self.assertEqual(b'\x01\x00\x02\x00', audio.resample(44100, 16000))

        audio = AudioFrames(b'', 2, 2)
        self.assertEqual(b'', audio.resample(44100, 16000))

        # Real audios files
        res_1 = AudioFrames(self.frames_1, self._sample_1.get_sampwidth()).resample(self._sample_1.get_framerate(), 44100)
        self.assertTrue(res_1.startswith(b'\x01\x00\x00\x00\xff\xff\xff\xff\xff\xff\x00\x00\x01\x00\x01\x00\x01\x00\x00\x00'))
        res_2 = AudioFrames(self.frames_2, self._sample_2.get_sampwidth()).resample(self._sample_2.get_framerate(), 44100)
        self.assertTrue(res_2.startswith(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x01\x00\x01\x00\x01\x00'))

    # -----------------------------------------------------------------------
    # Hopefully incoming: self-implemented resampling.
    # -----------------------------------------------------------------------

    def test_re_sample_valid_rates(self):
        # up-sampling: in-rate can't be greater than out-rate
        with self.assertRaises(NotImplementedError):
            AudioFrames._re_sample_valid_rates(8000, 16000)
        # down-sampling: supported out-rate is only 16kHz
        with self.assertRaises(NotImplementedError):
            AudioFrames._re_sample_valid_rates(48000, 12000)
        # down-sampling: supported in-rate is only a multiple of 16kHz
        with self.assertRaises(NotImplementedError):
            AudioFrames._re_sample_valid_rates(44100, 16000)

    # -----------------------------------------------------------------------

    def test_down_sample(self):
        # Not implemented:
        #   48kHz -> 32kHz => 3 samples are down-sampled into 2
        #   44100Hz -> 16kHz => 441 samples are down-sampled into 160
        with self.assertRaises(NotImplementedError):
            AudioFrames._down_sample([100, 200, -100], 2)

        # 16kHz -> 16kHz => 1 sample is down-sampled into 1, i.e. it's a deep copy!
        self.assertEqual([100], AudioFrames._down_sample([100], 1))
        self.assertEqual([100, 200], AudioFrames._down_sample([100, 200], 2))
        #
        # 32kHz -> 16kHz => 2 samples are down-sampled into 1
        self.assertEqual([150], AudioFrames._down_sample([100, 200], 1))
        self.assertEqual([66, 66], AudioFrames._down_sample([100, 200, -100, -200, 150, 250], 2))
        self.assertEqual([150, -150, 200], AudioFrames._down_sample([100, 200, -100, -200, 150, 250], 3))

        # 48kHz -> 16kHz => 3 samples are down-sampled into 1
        self.assertEqual([300], AudioFrames._down_sample([100, 200, 600], 1))

        # 96kHz -> 16kHz => 6 samples are down-sampled into 1
        self.assertEqual([300], AudioFrames._down_sample([100, 200, 600, 100, 200, 600], 1))

    # -----------------------------------------------------------------------

    def test_re_sample(self):
        frames = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00'
        audio_frames = AudioFrames(frames, 2, 1)
        frames8 = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00\x06\x00\x07\x00\x08\x00'
        audio_frames8 = AudioFrames(frames8, 2, 1)

        # not enough input frames
        self.assertEqual(b'', AudioFrames(b'', 2, 1)._re_sample(16000, 16000))
        self.assertEqual(b'\x01\x00', AudioFrames(b'\x01\x00', 2, 1)._re_sample(16000, 16000))
        self.assertEqual(b'\x01\x00', AudioFrames(b'\x01\x00', 2, 1)._re_sample(32000, 16000))
        self.assertEqual(b'\x01\x00', AudioFrames(b'\x01\x00', 2, 1)._re_sample(48000, 16000))
        self.assertEqual(b'\x03\x00', AudioFrames(b'\x02\x00\x04\x00', 2, 1)._re_sample(48000, 16000))
        self.assertEqual(b'\x03\x00', AudioFrames(b'\x02\x00\x04\x00', 2, 1)._re_sample(96000, 16000))

        # 16kHz -> 16kHz => 1 sample is down-sampled into 1 = deep copy.
        self.assertEqual(frames, audio_frames._re_sample(16000, 16000))

        # 32kHz -> 16kHz => 2 samples are down-sampled into 1
        self.assertEqual(b'\x01\x00\x03\x00\x05\x00\x07\x00', audio_frames8._re_sample(32000, 16000))
        #  -> the last buffer is not full:
        self.assertEqual(b'\x01\x00\x03\x00\x05\x00', audio_frames._re_sample(32000, 16000))

        # 48kHz -> 16kHz => 3 samples are down-sampled into 1
        #  -> the last buffer is not full:
        self.assertEqual(b'\x02\x00\x05\x00\x07\x00', audio_frames8._re_sample(48000, 16000))
        self.assertEqual(b'\x02\x00\x04\x00', audio_frames._re_sample(48000, 16000))

        # From real frames
        # oriana1.wav = 16kHz
        audio = AudioFrames(self.frames_1, 2, 1)
        self.assertEqual(self.frames_1, audio._re_sample(16000, 16000))

        # 48kHz -> 16kHz
        audio_sample_4 = audio_open(sample_4)
        frames_4 = audio_sample_4.read_frames(audio_sample_4.get_nframes())
        down_frames = AudioFrames(frames_4, 2, 1)._re_sample(48000, 16000)
        # to be tested.
        # audio_out = AudioPCM()
        # audio_out.append_channel(Channel(framerate=16000, sampwidth=2, frames=down_frames))
        # audio_save("test.wav", audio_out)

