# -*- coding:utf-8 -*-
"""
:filename: tests.test_ipus_searchfor.py
:author: Brigitte Bigi
:contact: contact@sppas.org
:summary: Test the full search for IPUs.

.. _This file is part of AudiooPy:
..

    Copyright (C) 2024 Brigitte Bigi
    Laboratoire Parole et Langage, Aix-en-Provence, France

    Use of this software is governed by the GNU Affero Public License, version 3.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program. If not, see <https://www.gnu.org/licenses/>.

    This banner notice must not be removed.

"""

import unittest
from unittest.mock import MagicMock
import os.path

from audioopy.channel import Channel
from audioopy.aio import open as audio_open
from audioopy.ipus import SearchForIPUs

sample_1 = os.path.join(os.path.dirname(os.path.abspath(__file__)), "samples", "oriana1.wav")
sample_3 = os.path.join(os.path.dirname(os.path.abspath(__file__)), "samples", "oriana3.wave")

# ---------------------------------------------------------------------------


class TestSearchForIPUs(unittest.TestCase):

    def test_init(self):
        channel = Channel(framerate=16000, sampwidth=2, frames=b"")
        searcher = SearchForIPUs(channel)
        self.assertEqual(searcher.get_win_length(), 0.02)
        self.assertEqual(searcher.get_vol_threshold(), 0)
        self.assertEqual(searcher.get_min_sil_dur(), 0.25)
        self.assertEqual(searcher.get_min_ipu_dur(), 0.3)
        self.assertEqual(searcher.get_shift_start(), 0.02)
        self.assertEqual(searcher.get_shift_end(), 0.02)

        # various valid win_len
        searcher = SearchForIPUs(channel, win_len=0.002)
        self.assertEqual(searcher.get_win_length(), 0.002)

        searcher = SearchForIPUs(channel, win_len=0.04)
        self.assertEqual(searcher.get_win_length(), 0.04)

        # invalid win_len
        searcher = SearchForIPUs(channel, win_len=0.5)
        self.assertEqual(searcher.get_win_length(), 0.05)

    def test_set_vol_threshold(self):
        channel = Channel(framerate=16000, sampwidth=2, frames=b"")
        search_ipus = SearchForIPUs(channel)

        # Valid value
        search_ipus.set_vol_threshold(100)
        self.assertEqual(search_ipus.get_vol_threshold(), 100)

        # Valid value for automatic search
        search_ipus.set_vol_threshold(0)
        self.assertEqual(search_ipus.get_vol_threshold(), 0)

        # Negative value
        search_ipus.set_vol_threshold(-10)
        self.assertEqual(search_ipus.get_vol_threshold(), SearchForIPUs.DEFAULT_VOL_THRESHOLD)

        # Invalid value
        with self.assertRaises(ValueError):
            search_ipus.set_vol_threshold("invalid")

    def test_set_min_sil(self):
        channel = Channel(framerate=16000, sampwidth=2, frames=b"")
        search_ipus = SearchForIPUs(channel)

        # Valid value
        search_ipus.set_min_sil(0.1)
        self.assertEqual(search_ipus._min_sil_dur, 0.1)

        # Too small value
        search_ipus.set_min_sil(0.01)
        self.assertEqual(search_ipus._min_sil_dur, SearchForIPUs.MIN_SIL_DUR)

        # Invalid value
        with self.assertRaises(ValueError):
            search_ipus.set_min_sil("invalid")

    def test_set_min_ipu(self):
        channel = Channel(framerate=16000, sampwidth=2, frames=b"")
        search_ipus = SearchForIPUs(channel)

        # Valid value
        search_ipus.set_min_ipu(0.1)
        self.assertEqual(search_ipus._min_ipu_dur, 0.1)

        # Too small value
        search_ipus.set_min_ipu(0.01)
        self.assertEqual(search_ipus._min_ipu_dur, SearchForIPUs.MIN_IPU_DUR)

        # Invalid value
        with self.assertRaises(ValueError):
            search_ipus.set_min_ipu("invalid")

    def test_set_shift_start(self):
        channel = Channel(framerate=16000, sampwidth=2, frames=b"")
        search_ipus = SearchForIPUs(channel)

        # Valid positive value
        search_ipus.set_shift_start(0.03)
        self.assertEqual(search_ipus.get_shift_start(), 0.03)

        # Valid negative value
        search_ipus.set_shift_start(-0.03)
        self.assertEqual(search_ipus.get_shift_start(), -0.03)

        # Too high pos value
        search_ipus = SearchForIPUs(channel)
        search_ipus.set_shift_start(0.4)
        self.assertEqual(search_ipus.get_shift_start(), 0.02)

        # Too high neg value
        search_ipus = SearchForIPUs(channel)
        search_ipus.set_shift_start(-0.4)
        self.assertEqual(search_ipus.get_shift_start(), 0.02)

        # Invalid value
        with self.assertRaises(ValueError):
            search_ipus.set_shift_start("invalid")

    def test_set_shift_end(self):
        channel = Channel(framerate=16000, sampwidth=2, frames=b"")
        search_ipus = SearchForIPUs(channel)

        # Valid positive value
        search_ipus.set_shift_end(0.03)
        self.assertEqual(search_ipus.get_shift_end(), 0.03)

        # Valid negative value
        search_ipus.set_shift_end(-0.03)
        self.assertEqual(search_ipus.get_shift_end(), -0.03)

        # Too high pos value
        search_ipus = SearchForIPUs(channel)
        search_ipus.set_shift_end(0.4)
        self.assertEqual(search_ipus.get_shift_end(), 0.02)

        # Too high neg value
        search_ipus = SearchForIPUs(channel)
        search_ipus.set_shift_end(-0.4)
        self.assertEqual(search_ipus.get_shift_end(), 0.02)

        # Invalid value
        with self.assertRaises(ValueError):
            search_ipus.set_shift_end("a")

    def test_min_channel_duration(self):
        channel = Channel(framerate=16000, sampwidth=2, frames=b"")
        search_ipus = SearchForIPUs(channel)
        expected_duration = 0.300 + 0.02 + 0.02
        self.assertEqual(search_ipus.min_channel_duration(), expected_duration)

    def test_get_channel(self):
        channel = Channel(framerate=16000, sampwidth=2, frames=b"\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00\x06\x00")
        searcher = SearchForIPUs(channel)
        self.assertEqual(searcher.get_channel().get_frames(), b"\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00\x06\x00")

    def test_get_rms_stats(self):
        channel = Channel(framerate=16000, sampwidth=2, frames=b"")
        searcher = SearchForIPUs(channel)
        searcher._ChannelSilences__volume_stats._volumes = [0.1, 0.2, 0.3, 0.4, 0.5]
        stats = searcher.get_rms_stats()
        self.assertEqual(stats[0], 0.1)  # min
        self.assertEqual(stats[1], 0.5)  # max
        self.assertAlmostEqual(stats[2], 0.3)  # mean
        self.assertAlmostEqual(stats[3], 0.3)  # median
        self.assertAlmostEqual(stats[4],  0.00471, places=5)  # coefvariation

    def test_ipus_searchfor(self):
        channel = Channel(framerate=16000, sampwidth=2, frames=b"\x01\x02" * 100)
        searcher = SearchForIPUs(channel)
        # Set a high threshold to ensure no silences are detected
        searcher.set_vol_threshold(10000)
        tracks = searcher.get_tracks()
        # so the whole channel is a track
        self.assertEqual(tracks, [(0, 100.0)])

        a1 = audio_open(sample_1)
        # Extract channel
        a1.extract_channel(0)
        # Search for IPUs with default values
        searcher = SearchForIPUs(a1[0], win_len=0.05)

        tracks = searcher.get_tracks(time_domain=True)
        expected_tracks = [(1.6675, 4.52), (6.38, 9.52), (11.4175, 14.62)]
        self.assertEqual(tracks, expected_tracks)

    def test_tracks_data(self):
        channel = MagicMock(spec=Channel)
        channel.get_nframes.return_value = 1000
        search_ipus = SearchForIPUs(channel)

        # Valid track
        channel.get_frames.return_value = b'frames_data'
        tracks = [(0, 100), (200, 300), (400, 500)]
        result = list(search_ipus._SearchForIPUs__track_data(tracks))
        self.assertEqual(result, [b'frames_data', b'frames_data', b'frames_data'])

        # Empty tracks list
        channel = MagicMock(spec=Channel)
        search_ipus = SearchForIPUs(channel)
        tracks = []
        result = list(search_ipus._SearchForIPUs__track_data(tracks))
        self.assertEqual(result, [])

        # Invalid tracks
        invalid_tracks = [(0,), (200, '300'), (400, 500, 600)]
        with self.assertRaises(TypeError):
            list(search_ipus._SearchForIPUs__track_data(invalid_tracks))
