"""
:filename: tests.test_channelsilence.py
:author: Brigitte Bigi
:contact: contact@sppas.org
:summary: Test of the channel silence detection.

.. _This file is part of AudiooPy:
..
    ---------------------------------------------------------------------

    Copyright (C) 2024 Brigitte Bigi
    Laboratoire Parole et Langage, Aix-en-Provence, France

    Use of this software is governed by the GNU Affero Public License, version 3.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program. If not, see <https://www.gnu.org/licenses/>.

    This banner notice must not be removed.

    ---------------------------------------------------------------------

"""

import unittest
from unittest.mock import MagicMock
import os
import struct

import audioopy.aio
from audioopy.channel import Channel
from audioopy.channelvolume import ChannelVolume
from audioopy.ipus.channelsilences import ChannelSilences

# ---------------------------------------------------------------------------

# English language -- recommended min sil duration = 0.250 seconds
sample_1 = os.path.join(os.path.dirname(os.path.abspath(__file__)), "samples", "oriana1.wav")
# French language -- recommended min sil duration = 0.200 seconds
sample_2 = os.path.join(os.path.dirname(os.path.abspath(__file__)), "samples", "F_F_B003_P8.wav")

# ---------------------------------------------------------------------------


class TestSilences(unittest.TestCase):
    """Test the search of silences."""

    def setUp(self):
        # Create the samples.
        # with framerate=8000, our 8000 samples are representing 1 second.
        # 1 sample represents 0.000125 second.
        samples = [0] * 8000

        for i in range(2000, 3000):
            if i % 2:
                samples[i] = i - 2000
            else:
                samples[i] = -i + 2000

        for i in range(3000, 5000):
            if i % 2:
                samples[i] = 1000
            else:
                samples[i] = -1000

        for j, i in enumerate(range(5000, 6000)):
            if i % 2:
                samples[i] = 1000 - j
            else:
                samples[i] = -1000 + j

        # Convert samples into frames: it divides the use of memory by 2 -- only!
        converted = b''.join(struct.pack('<h', elem) for elem in samples)

        self.channel = Channel(framerate=8000, sampwidth=2, frames=converted)

    # -----------------------------------------------------------------------

    def test_set_win_len(self):
        silences = ChannelSilences(self.channel, win_len=0.020, vagueness=0.005)

        # Valid positive value
        silences.set_win_length(0.03)
        self.assertEqual(silences.get_win_length(), 0.03)

        # Too high pos value
        silences = ChannelSilences(self.channel, win_len=0.020, vagueness=0.005)
        silences.set_win_length(0.4)
        self.assertEqual(silences.get_win_length(), ChannelSilences.MAX_WIN_LEN)

        # Negative value
        silences = ChannelSilences(self.channel, win_len=0.020, vagueness=0.005)
        silences.set_win_length(-0.4)
        self.assertEqual(silences.get_win_length(), ChannelSilences.MIN_WIN_LEN)

        # Invalid value
        with self.assertRaises(ValueError):
            silences.set_win_length("a")

    # -----------------------------------------------------------------------

    def test_vagueness(self):
        silences = ChannelSilences(self.channel, win_len=0.020, vagueness=0.005)
        self.assertEqual(0.005, silences.get_vagueness())

        # Valid value
        silences.set_vagueness(0.01)
        self.assertEqual(0.01, silences.get_vagueness())

        # Too high value
        silences.set_vagueness(0.05)
        self.assertEqual(0.02, silences.get_vagueness())

        # Negative value
        silences.set_vagueness(-0.005)
        self.assertEqual(0., silences.get_vagueness())

        # Invalid value
        with self.assertRaises(ValueError):
            silences.set_vagueness("a")

    # -----------------------------------------------------------------------

    def test_set_channel(self):
        silences = ChannelSilences(self.channel, win_len=0.020, vagueness=0.005)
        self.assertEqual(self.channel, silences._channel)

        # Valid instance
        cha = Channel()
        silences.set_channel(cha)
        self.assertEqual(cha, silences._channel)

        # Invalid channel
        with self.assertRaises(TypeError):
            silences.set_channel("invalid")

    # -----------------------------------------------------------------------

    def test_set_silences(self):
        channel_silences = ChannelSilences(self.channel, win_len=0.020, vagueness=0.005)

        # Valid list of tuples
        silences = [(1000, 2000), (3000, 4000), (5000, 6000)]
        channel_silences.set_silences(silences)
        self.assertEqual(channel_silences._ChannelSilences__silences, silences)

        # Invalid type
        with self.assertRaises(TypeError):
            channel_silences.set_silences("not a list")

        # Invalid list with non-tuple elements
        with self.assertRaises(TypeError):
            silences = [(1000, 2000), "not a tuple", (5000, 6000)]
            channel_silences.set_silences(silences)

    # -----------------------------------------------------------------------

    def test_reset_silences(self):
        channel_silences = ChannelSilences(self.channel, win_len=0.020, vagueness=0.005)
        silences = [(1000, 2000), (3000, 4000), (5000, 6000)]
        channel_silences.set_silences(silences)

        channel_silences.reset_silences()
        self.assertEqual(channel_silences._ChannelSilences__silences, [])

    # -----------------------------------------------------------------------

    def test_get_volstats(self):
        silences = ChannelSilences(self.channel, win_len=0.020, vagueness=0.005)
        # there are 160 samples in a window of 20ms.
        # so we'll estimate 8000/160 = 50 rms values
        rms = silences.get_volstats()
        self.assertEqual(50, len(rms))
        # The 2000 first and last samples are 0. so rms is also 0.
        for i in range(12):
            self.assertEqual(0, rms[i])
            self.assertEqual(0, rms[50-i-1])
        # Others are positive values
        for i in range(13, 50-12):
            self.assertGreater(rms[i], 0)

    # -----------------------------------------------------------------------

    def test_fix_threshold_vol(self):
        silences = ChannelSilences(self.channel, win_len=0.020, vagueness=0.005)
        threshold = silences.fix_threshold_vol()
        self.assertEqual(376, threshold)

    # -----------------------------------------------------------------------

    def test_search_silences(self):
        silences = ChannelSilences(self.channel, win_len=0.020, vagueness=0.005)
        threshold = silences.fix_threshold_vol()
        th = silences.search_silences(threshold)
        self.assertEqual(th, threshold)
        # we have to found 2 silences: at the beginning and at the end
        self.assertEqual(2, len(silences))
        self.assertEqual((0, 2240), silences[0])
        self.assertEqual((5600, 8000), silences[1])

    # -----------------------------------------------------------------------

    def test_extract_tracks(self):
        silences = ChannelSilences(self.channel, win_len=0.020, vagueness=0.005)
        silences.search_silences()
        self.assertEqual(2, len(silences))
        tracks = silences.extract_tracks(min_track_dur=0.2, shift_dur_start=0., shift_dur_end=0.)
        self.assertEqual(1, len(tracks))
        self.assertEqual((2240, 5600), tracks[0])

# ---------------------------------------------------------------------------


class TestIntegrationSilences(unittest.TestCase):
    """Test the search of silences on actual audio files."""

    def setUp(self):
        # Extract the only channel of the English sample
        audio = audioopy.aio.open(sample_1)
        audio.extract_channel(0)
        self.channel_1 = audio[0]
        # Extract the only channel of the French sample
        audio = audioopy.aio.open(sample_2)
        audio.extract_channel(0)
        self.channel_2 = audio[0]

    def test_full_tracks_detection_sample_1(self):
        # Create ChannelSilences() instance: estimate the rms values in windows of 20 ms length
        channel_silences = ChannelSilences(self.channel_1, win_len=0.02, vagueness=0.005)

        # Search for all the silences, comparing each rms to an automatically estimated threshold
        threshold = channel_silences.search_silences()
        expected_silences = [(0, 26880), (27840, 29440), (36480, 37440), (50240, 51200), (52800, 56640), (63680, 65600), (71040, 102080), (111680, 113280), (120640, 121600), (124480, 127040), (128320, 132160), (137280, 138560), (151680, 182720), (193280, 194560), (196480, 197440), (199360, 200320), (204800, 206400), (209280, 214080), (216640, 217600), (218240, 221120), (224640, 225600), (227520, 230080), (231360, 284672.0)]
        self.assertEqual(list(channel_silences), expected_silences)

        # Keep only silences during more than a given duration
        channel_silences.filter_silences(threshold // 2, 0.250)
        expected_silences = [(0, 27040), (72000, 102400), (152000, 183040), (231520, 284672.0)]
        self.assertEqual(list(channel_silences), expected_silences)

        # Get the (from_pos, to_pos) of the tracks during more than a given duration
        tracks = channel_silences.extract_tracks(0.300, 0.02, 0.02)
        expected_tracks = [(26720, 72320), (102080, 152320), (182720, 231840)]
        self.assertEqual(list(tracks), expected_tracks)

    def test_full_tracks_detection_sample_2(self):
        # Create ChannelSilences() instance: estimate the rms values in windows of 20 ms length
        channel_silences = ChannelSilences(self.channel_2, win_len=0.02, vagueness=0.005)

        # Search for all the silences, comparing each rms to an automatically estimated threshold
        threshold = channel_silences.search_silences()
        expected_silences = [
            (0, 64386), (66150, 110250), (150822, 153468), (156996, 161406), (169344, 173754),
            (177282, 180810), (187866, 192276), (195804, 198450), (201978, 214326), (216972, 224028),
            (232848, 239022), (244314, 252252), (267246, 270774), (274302, 280476), (288414, 292824),
            (297234, 303408), (309582, 313110), (340452, 344862), (358092, 365148), (369558, 404838),
            (415422, 418068), (424242, 427770), (431298, 433944), (437472, 441000), (444528, 447174),
            (451584, 454230), (463932, 466578), (478926, 482454), (485982, 491274), (496566, 500094),
            (523908, 553014), (578592, 583002), (591822, 595350), (600642, 613872), (620928, 625338),
            (633276, 635922), (657090, 732060), (735588, 739116), (747936, 754992), (756756, 759402),
            (769986, 779688), (791154, 793800), (807030, 811440), (818496, 823788), (832608, 837018),
            (838782, 844956), (852894, 857304), (859950, 864360), (866124, 870534), (874062, 941408.0)]
        self.assertEqual(list(channel_silences), expected_silences)

        # Keep only silences during more than a given duration
        channel_silences.filter_silences(threshold // 2, 0.200)
        expected_silences = [
            (0, 65263), (69007, 111127), (202855, 213003), (370435, 406155), (525445, 554111),
            (601299, 614529), (658187, 732497), (771083, 780565), (874719, 941408.0)]
        self.assertEqual(list(channel_silences), expected_silences)

        # Get the (from_pos, to_pos) of the tracks during more than a given duration
        tracks = channel_silences.extract_tracks(0.300, 0.02, 0.02)
        expected_tracks = [(110245, 203737), (212121, 371317), (405273, 526327), (553229, 602181),
                           (613647, 659069), (731615, 771965), (779683, 875601)]
        self.assertEqual(list(tracks), expected_tracks)
