"""
:filename: tests.test_volume.py
:author: Brigitte Bigi
:contact: contact@sppas.org
:summary: Test of the volumes of a channel or an audio.

.. _This file is part of AudiooPy:
..
    ---------------------------------------------------------------------

    Copyright (C) 2024 Brigitte Bigi
    Laboratoire Parole et Langage, Aix-en-Provence, France

    Use of this software is governed by the GNU Affero Public License, version 3.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program. If not, see <https://www.gnu.org/licenses/>.

    This banner notice must not be removed.

    ---------------------------------------------------------------------

"""

import unittest
import os.path
import statistics
from unittest.mock import MagicMock

from audioopy.aio import open as audio_open
from audioopy.channel import Channel
from audioopy.basevolume import BaseVolume
from audioopy.channelvolume import ChannelVolume
from audioopy.audiovolume import AudioVolume

# ---------------------------------------------------------------------------

# mono; 16000Hz; 16bits
sample_1 = os.path.join(os.path.dirname(os.path.abspath(__file__)), "samples", "oriana1.wav")

# ---------------------------------------------------------------------------


class TestBaseVolume(unittest.TestCase):

    def test_init(self):
        """Test init BaseVolume."""
        bv = BaseVolume()
        self.assertEqual(bv._volumes, [])
        self.assertEqual(bv._rms, 0.0)
        self.assertEqual(bv._winlen, 0.01)

        # Valid win_len value
        bv = BaseVolume(win_len=0.02)
        self.assertEqual(bv._winlen, 0.02)

        # Invalid win_len value
        bv = BaseVolume(win_len=1e-10)
        self.assertEqual(bv._winlen, 0.01)

        with self.assertRaises(ValueError):
            BaseVolume(win_len="a")

    def test_volume(self):
        """Test BaseVolume rms getter."""
        bv = BaseVolume(win_len=0.02)
        self.assertEqual(bv.volume(), 0.0)

        bv._rms = 0.3162
        self.assertAlmostEqual(bv.volume(), 0.3162, places=4)

    def test_volumes(self):
        """Test BaseVolume volumes getter."""
        bv = BaseVolume(win_len=0.02)
        self.assertEqual(bv.volumes(), [])
        bv._volumes = [0.1, 0.2, 0.3, 0.4, 0.5]
        self.assertEqual(bv.volumes(), [0.1, 0.2, 0.3, 0.4, 0.5])

    def test_volume_at(self):
        """Test BaseVolume volume_at."""
        # single volume value
        bv = BaseVolume(win_len=0.02)
        bv._volumes = [0.1]
        index = 0
        self.assertEqual(bv.volume_at(index), 0.1)
        self.assertIsInstance(bv.volume_at(index), float)

        # multiple volume values, positive index
        bv = BaseVolume(win_len=0.02)
        bv._volumes = [0.1, 0.2, 0.3, 0.4, 0.5]
        index = 3
        self.assertEqual(bv.volume_at(index), 0.4)
        self.assertIsInstance(bv.volume_at(index), float)

        # multiple volume values, negative index
        bv = BaseVolume(win_len=0.02)
        bv._volumes = [0.1, 0.2, 0.3, 0.4, 0.5]
        index = -1
        self.assertEqual(bv.volume_at(index), 0.5)

        # invalid index
        bv._volumes = []
        index = 0
        with self.assertRaises(IndexError):
            bv.volume_at(index)

        bv._volumes = [0.1, 0.2, 0.3, 0.4, 0.5]
        index = -10
        with self.assertRaises(IndexError):
            bv.volume_at(index)

    def test_min(self):
        """Test BaseVolume minimum volume."""
        bv = BaseVolume(win_len=0.02)
        bv._volumes = [0.1, 0.2, 0.3, 0.4, 0.5]
        self.assertEqual(bv.min(), 0.1)

        bv = BaseVolume(win_len=0.02)
        bv._volumes = []
        self.assertEqual(bv.min(), 0.)

    def test_max(self):
        """Test BaseVolume maximum volume."""
        bv = BaseVolume(win_len=0.02)
        bv._volumes = [0.1, 0.2, 0.3, 0.5, 0.5]
        self.assertEqual(bv.max(), 0.5)

        bv = BaseVolume(win_len=0.02)
        bv._volumes = []
        self.assertEqual(bv.max(), 0.)

    def test_mean(self):
        """Test BaseVolume mean volume."""
        bv = BaseVolume(win_len=0.02)
        bv._volumes = [0.1, 0.2, 0.3, 0.4, 0.5]
        self.assertEqual(bv.mean(), 0.3)
        self.assertEqual(bv.mean(), statistics.mean(bv._volumes))

        bv = BaseVolume(win_len=0.02)
        bv._volumes = []
        self.assertEqual(bv.mean(), 0.)

    def test_median(self):
        """Test BaseVolume median volume."""
        bv = BaseVolume(win_len=0.02)
        bv._volumes = [0.1, 0.2, 0.3, 0.4, 0.5]
        self.assertEqual(bv.median(), 0.3)
        self.assertEqual(bv.median(), statistics.median(bv._volumes))

        bv = BaseVolume(win_len=0.02)
        bv._volumes = []
        self.assertEqual(bv.median(), 0.)

    def test_variance(self):
        """Test BaseVolume variance volume."""
        bv = BaseVolume(win_len=0.02)
        bv._volumes = [0.1, 0.2, 0.3, 0.4, 0.5]
        self.assertAlmostEqual(bv.variance(), 0.02, places=4)

        bv = BaseVolume(win_len=0.02)
        bv._volumes = []
        self.assertEqual(bv.variance(), 0.)

    def test_std(self):
        """Test BaseVolume standard deviation volume."""
        bv = BaseVolume(win_len=0.02)
        bv._volumes = [0.1, 0.2, 0.3, 0.4, 0.5]
        self.assertAlmostEqual(bv.stdev(), 0.1414, places=4)

        bv = BaseVolume(win_len=0.02)
        bv._volumes = []
        self.assertEqual(bv.stdev(), 0.)

    def test_coefvariation(self):
        """Test BaseVolume coefficient of variation volume."""
        bv = BaseVolume(win_len=0.02)
        bv._volumes = [0.1, 0.2, 0.3, 0.4, 0.5]
        result = bv.coefvariation()
        expected_result = bv.stdev() / (bv.mean() * 100.)
        self.assertAlmostEqual(result, expected_result, places=6)

        bv = BaseVolume(win_len=0.02)
        bv._volumes = []
        result = bv.coefvariation()
        self.assertEqual(result, 0.)

        bv = BaseVolume(win_len=0.02)
        bv._volumes = [0.0001, 0.001, 0.001, 0.00001, 0.0001]
        result = bv.coefvariation()
        self.assertEqual(result, 0.)

    def test_zscores(self):
        """Test BaseVolume z-scores of volumes."""
        bv = BaseVolume(win_len=0.02)
        bv._volumes = [0.1, 0.2, 0.3, 0.4, 0.5]
        expected_zscores = [-1.414213562373095, -0.7071067811865474, 0.0, 0.7071067811865478, 1.4142135623730951]
        self.assertEqual(bv.zscores(), expected_zscores)

        bv = BaseVolume(win_len=0.02)
        bv._volumes = []
        self.assertEqual(bv.zscores(), [])

    def test_stderr(self):
        """Test BaseVolume standard error of volumes."""
        bv = BaseVolume(win_len=0.02)
        bv._volumes = [0.1, 0.2, 0.3, 0.4, 0.5]
        self.assertAlmostEqual(bv.stderr(), 0.0632, places=4)

        bv = BaseVolume(win_len=0.02)
        bv._volumes = []
        self.assertEqual(bv.stderr(), 0.)

# ---------------------------------------------------------------------------


class TestChannelVolume(unittest.TestCase):

    def test_init(self):
        # Mock the Channel object
        mock_channel = MagicMock(spec=Channel)
        mock_channel.tell.return_value = 0
        mock_channel.get_framerate.return_value = 16000
        mock_channel.get_duration.return_value = 1.0
        mock_channel.get_frames.return_value = b'\x01\x00\x02\x00\x03\x00\x04\x00\x05\x00\x06\x00'
        mock_channel.get_sampwidth.return_value = 2
        mock_channel.rms.return_value = 0.5

        channel_volume = ChannelVolume(mock_channel)
        self.assertEqual(channel_volume._win_len, 0.01)
        self.assertEqual(channel_volume._channel, mock_channel)
        self.assertEqual(channel_volume._rms, 0.5)

    def test_evaluate_rms(self):
        channel = Channel(framerate=48000, sampwidth=2, frames=b"")
        volumes = ChannelVolume.calculate_volumes(channel, win_len=0.02)
        self.assertEqual(volumes, [])

        channel = Channel(framerate=16000, sampwidth=2, frames=b'\x01\x00\x02\x00\x03\x00\x04\x00')
        volumes = ChannelVolume.calculate_volumes(channel, win_len=0.01)
        self.assertEqual(volumes, [2.74])

    def test_normalize_volumes(self):
        # Mock Channel object
        channel = Channel(framerate=48000, sampwidth=2, frames=b'\x01\x00\x02\x00\x03\x00\x04\x00')
        channel_volume = ChannelVolume(channel, win_len=0.01)

        # Normal case
        channel_volume._volumes = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
        outliers_removed = channel_volume.normalize_volumes()
        self.assertEqual(channel_volume._volumes, [1, 2, 3, 4, 5, 6, 7, 8, 9, 9])
        self.assertEqual(outliers_removed, 1)

        # Empty list
        channel_volume._volumes = []
        outliers_removed = channel_volume.normalize_volumes()
        self.assertEqual(channel_volume._volumes, [])
        self.assertEqual(outliers_removed, 0)

        # Nothing to do
        channel_volume._volumes = [5, 5, 5, 5, 5]
        outliers_removed = channel_volume.normalize_volumes()
        self.assertEqual(channel_volume._volumes, [5, 5, 5, 5, 5])
        self.assertEqual(outliers_removed, 0)

# ---------------------------------------------------------------------------


class TestVolume(unittest.TestCase):

    def test_rms(self):
        audio = audio_open(sample_1)
        cidx = audio.extract_channel(0)
        channel = audio.get_channel(cidx)

        chanvol = ChannelVolume(channel)
        audiovol = AudioVolume(audio)

        self.assertEqual(chanvol.volume(), channel.rms())
        self.assertEqual(audiovol.volume(), audio.rms())
        self.assertEqual(chanvol.len(), int(channel.get_duration()/0.01) + 1)
        self.assertEqual(chanvol.min(), audiovol.min())
        self.assertEqual(chanvol.max(), audiovol.max())
        self.assertEqual(int(chanvol.mean()), int(audiovol.mean()))
        self.assertEqual(int(chanvol.variance()), int(audiovol.variance()))
        self.assertEqual(int(chanvol.stdev()), int(audiovol.stdev()))

        self.assertEqual(chanvol.min(), 1.61)
        self.assertAlmostEqual(chanvol.mean(), 360.12, places=2)
        self.assertAlmostEqual(chanvol.median(), 71.93, places=2)
        self.assertAlmostEqual(chanvol.stdev(), 595.43, places=2)
