# -*- coding:utf-8 -*-
"""
:filename: tests.test_waveio.py
:author: Brigitte Bigi
:contact: contact@sppas.org
:summary: Test the WaveIO class

.. _This file is part of AudiooPy:
..
    ---------------------------------------------------------------------

    Copyright (C) 2024 Brigitte Bigi
    Laboratoire Parole et Langage, Aix-en-Provence, France

    Use of this software is governed by the GNU Affero Public License, version 3.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program. If not, see <https://www.gnu.org/licenses/>.

    This banner notice must not be removed.

    ---------------------------------------------------------------------

"""

import time
import wave
import unittest
import unittest.mock as mock

from audioopy.aio import WaveIO
from audioopy.channel import Channel

# ---------------------------------------------------------------------------


class TestWaveIO(unittest.TestCase):

    def test_init(self):
        pass

    def test_open(self):
        """Test WaveIO open an audio."""

        # mock an empty channel
        mock_wave_open = mock.MagicMock()
        mock_wave_open.getnchannels.return_value = 2
        mock_wave_open.getsampwidth.return_value = 2
        mock_wave_open.getframerate.return_value = 48000
        mock_wave_open.getnframes.return_value = 1000

        with mock.patch('wave.open', return_value=mock_wave_open):
            wave_io = WaveIO()
            wave_io.open("valid_audio.wav")

            self.assertEqual(wave_io.get_nchannels(), 2)
            self.assertEqual(wave_io.get_sampwidth(), 2)
            self.assertEqual(wave_io.get_framerate(), 48000)
            self.assertEqual(wave_io.get_nframes(), 1000)

        # add frames to the channel
        mock_wave_open.readframes.return_value = b'\x00\x01\x02\x03' * 500
        with mock.patch('wave.open', return_value=mock_wave_open):
            wave_io = WaveIO()
            wave_io.open("valid_audio.wav")
            channel_index = wave_io.extract_channel(0)

            self.assertEqual(channel_index, 0)
            self.assertIsInstance(wave_io.get_channel(0), Channel)

        # non-existing audio file
        with mock.patch('wave.open', side_effect=wave.Error):
            wave_io = WaveIO()
            with self.assertRaises(wave.Error):
                wave_io.open("missing_audio.wav")

    def test_save(self):
        """Test WaveIO save an audio."""

        # One channel -- mono
        mock_channel = mock.MagicMock()
        mock_channel.get_sampwidth.return_value = 2
        mock_channel.get_framerate.return_value = 48000
        mock_channel.get_frames.return_value = b'\x00\x01' * 1000

        wave_io = WaveIO()
        wave_io.append_channel(mock_channel)

        with mock.patch('wave.Wave_write') as mock_wave_write:
            wave_io.save("output.wav")
            mock_wave_write.assert_called_once_with("output.wav")

        # Two channels -- stereo
        mock_channel2 = mock.MagicMock()
        mock_channel2.get_sampwidth.return_value = 2
        mock_channel2.get_framerate.return_value = 48000
        mock_channel2.get_frames.return_value = b'\x02\x03' * 1000

        wave_io = WaveIO()
        wave_io.append_channel(mock_channel)
        wave_io.append_channel(mock_channel2)

        with mock.patch('wave.Wave_write') as mock_wave_write:
            wave_io.save("output.wav")
            mock_wave_write.assert_called_once_with("output.wav")

    def test_save_fragment(self):
        """Test WaveIO save fragment of an audio."""
        wave_io = WaveIO()
        wave_io._audio_fp = mock.MagicMock()
        wave_io._audio_fp.getnchannels.return_value = 1
        wave_io._audio_fp.getsampwidth.return_value = 2
        wave_io._audio_fp.getframerate.return_value = 48000

        # a few frames only
        frames = b'\x00\x01' * 1000
        with mock.patch('wave.open', mock.mock_open()) as mocked_wave_open:
            try:
                wave_io.save_fragment("output.wav", frames)
            except Exception as e:
                self.fail(f"save_fragment raised an exception {e}")
        frames = b'\x00\x01' * (10**7)  # Large frame data

        # large amount of frames
        with mock.patch('wave.open', mock.mock_open()) as mocked_wave_open:
            start_time = time.time()
            wave_io.save_fragment("output.wav", frames)
            end_time = time.time()
            # Ensure it completes within 5 seconds
            self.assertLess(end_time - start_time, 5)

        # invalid frames
        frames = "not bytes"
        with self.assertRaises(TypeError):
            wave_io.save_fragment("output.wav", frames)